library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;

entity NF1 is
    port( CLK:  in   std_logic;
          LED0, LED1, LED2, LED3,
          LED4, LED5, LED6, LED7:  out   std_logic );
end NF1;

architecture RTL of NF1 is

component TIMER is
   generic( CYCLE: integer:=13500000); -- default 0.5s
   port( CLK:     in   std_logic;
          PULSE :  out  std_logic );
end component;

component BIN4 is
     port( SW0:   in   std_logic;
           LED0, LED1, LED2, LED3: out  std_logic );
end component;

signal	TOUT:  std_logic;
signal	INDAT: std_logic_vector( 3 downto 0 ):="0000";
signal	LEDS:  std_logic_vector( 7 downto 0 );

begin
U01: TIMER generic map( CYCLE=>6750000 ) -- 250 ms
           port map( CLK=>CLK, PULSE=>TOUT );

U02: BIN4 port map( SW0=>TOUT, LED0=>INDAT(0),
     LED1=>INDAT(1), LED2=>INDAT(2), LED3=>INDAT(3) ); 
	 
    LED0 <= LEDS(0); LED1 <= LEDS(1);
    LED2 <= LEDS(2); LED3 <= LEDS(3);   
    LED4 <= LEDS(4); LED5 <= LEDS(5);
    LED6 <= LEDS(6); LED7 <= LEDS(7); 

process( INDAT )
    begin
        case INDAT is
            when "0000"  => LEDS <= "00000001";
            when "0001"  => LEDS <= "00000010";
            when "0010"  => LEDS <= "00000100";
            when "0011"  => LEDS <= "00001000";
            when "0100"  => LEDS <= "00010000";
            when "0101"  => LEDS <= "00100000";
            when "0110"  => LEDS <= "01000000";
            when "0111"  => LEDS <= "10000000";
            when "1000"  => LEDS <= "10000000";
            when "1001"  => LEDS <= "01000000";
            when "1010"  => LEDS <= "00100000";
            when "1011"  => LEDS <= "00010000";
            when "1100"  => LEDS <= "00001000";
            when "1101"  => LEDS <= "00000100";
            when "1110"  => LEDS <= "00000010";
            when others  => LEDS <= "00000001";
        end case;
    end process;
end RTL;
