library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;

entity NF2 is
   generic( MODE: string(1 to 3):= "RTL");
   port( CLK:  in   std_logic;
         SW3:  in   std_logic;
         LED0, LED1, LED2, LED3, LED4,
         LED5, LED6, LED7: out std_logic );
end NF2;

architecture RTL of NF2 is

component TIMER is
   generic( CYCLE: integer :=13500000); -- default 0.5s
   port( CLK:    in   std_logic;
         PULSE : out  std_logic );
end component;

component CNT16ARUD is
   port( SW0:   in   std_logic;
         SW1:   in   std_logic;  -- Up/Down
         SW3:   in   std_logic;  -- RESET
         LED0, LED1, LED2, LED3:  out  std_logic );
end component;

signal  TOUT:  std_logic;
signal  INDAT: std_logic_vector( 3 downto 0 );
signal  LEDS:  std_logic_vector( 7 downto 0 );
signal  UD:    std_logic:='1';

begin

TM0: if MODE = "RTL" generate
U01: TIMER generic map( CYCLE=>6750000 )  -- 250 ms
           port map( CLK=>CLK, PULSE=>TOUT );
end generate;

TM1: if MODE = "SIM" generate
U01: TIMER generic map( CYCLE=>6750000 )  -- 250 ms
           port map( CLK=>CLK, PULSE=>open );
     TOUT <= CLK;
end generate;

U02: CNT16ARUD port map( SW0=>TOUT, SW1=>UD, SW3=>SW3,
         LED0=>INDAT(0), LED1=>INDAT(1), LED2=>INDAT(2),
         LED3=>INDAT(3) ); 
	 
   LED0 <= LEDS(0); LED1 <= LEDS(1);
   LED2 <= LEDS(2); LED3 <= LEDS(3);
   LED4 <= LEDS(4); LED5 <= LEDS(5);
   LED6 <= LEDS(6); LED7 <= LEDS(7);

   process( SW3, INDAT )      --Up/Down control
   begin
      if   ( SW3='0' )      then UD<='1';  --RESET, UP
      elsif( INDAT="0111" ) then UD<='0'; -- Down
      elsif( INDAT="0000" ) then UD<='1'; -- Up
      end if;
        end process;
	 
   process( INDAT )
   begin
      case INDAT is
         when "0000"  => LEDS <= "00000001";
         when "0001"  => LEDS <= "00000010";
         when "0010"  => LEDS <= "00000100";
         when "0011"  => LEDS <= "00001000";
         when "0100"  => LEDS <= "00010000";
         when "0101"  => LEDS <= "00100000";
         when "0110"  => LEDS <= "01000000";
         when "0111"  => LEDS <= "10000000";
         when others  => LEDS <= "11111111";
      end case;
   end process;
end RTL;
--★本書では ; と記載がありますが，誤りです。削除してくださいますよう，お願い申し上げます。
