library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;

entity NF3 is
    generic( MODE: string(1 to 3):= "RTL");
    port( CLK: in  std_logic;
          SW3: in  std_logic; -- RESET
          LED0, LED1, LED2, LED3,
          LED4, LED5, LED6, LED7: out   std_logic );
end NF3;

architecture RTL of NF3 is

component TIMER is
    generic( CYCLE: integer :=13500000);  -- default 0.5s
    port( CLK:    in   std_logic;
          PULSE : out  std_logic );
end component;

signal  TOUT:  std_logic;
signal  LEDS:  std_logic_vector( 7 downto 0 );
signal  LR:    std_logic:='0';

begin
TM0: if MODE = "RTL" generate
U01: TIMER generic map( CYCLE=>6750000 )  -- set 250 ms
           port map( CLK=>CLK, PULSE=>TOUT );
end generate;

TM1: if MODE = "SIM" generate
U01: TIMER generic map( CYCLE=>6750000 )  -- set 250 ms
           port map( CLK=>CLK, PULSE=>open );
     TOUT <= CLK;
end generate;

    LED0 <= LEDS(0); LED1 <= LEDS(1);
    LED2 <= LEDS(2); LED3 <= LEDS(3);
    LED4 <= LEDS(4); LED5 <= LEDS(5);
    LED6 <= LEDS(6); LED7 <= LEDS(7);

   process( SW3, LEDS )       -- Shift Direction control
   begin
      if   ( SW3='0' )       then LR <= '0'; -- RESET
      elsif( LEDS(7) = '1' ) then LR <= '0'; --move right
      elsif( LEDS(0) = '1' ) then LR <= '1'; --move right
      end if;
   end process;

   process( SW3, TOUT ) 
   begin
      if( SW3='0' )   then LEDS <= "00000001"; -- RESET
      elsif( TOUT'event and TOUT='0' ) then
         if( LR='0' ) then LEDS <= LEDS(0) & LEDS(7 downto 1);
         else              LEDS <= LEDS(6 downto 0) & LEDS(7);
         end if;
      end if;
   end process;
end RTL;
