library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;

entity NF_ROM is
    port( CLK:  in   std_logic;
          LED0, LED1, LED2, LED3,
          LED4, LED5, LED6, LED7:  out   std_logic );
end NF_ROM;

architecture RTL of NF_ROM is
component TIMER is
    generic( CYCLE: integer:=13500000); -- default 0.5s
    port( CLK:     in   std_logic;
          PULSE :  out  std_logic );
end component;

component BIN4 is
     port( SW0:  in   std_logic;
           LED0, LED1, LED2, LED3: out  std_logic );
end component;

signal   TOUT:  std_logic;
signal   INDAT: std_logic_vector( 3 downto 0 );
signal   LEDS:  std_logic_vector( 7 downto 0 );

type ROMARRAY is array(0 to 15) of std_logic_vector( 7 downto 0 );
constant LEDPAT: ROMARRAY := (
        "00000001", "00000010", "00000100", "00001000",
        "00010000", "00100000", "01000000", "10000000",
        "10000000", "01000000", "00100000", "00010000",
        "00001000", "00000100", "00000010", "00000001");
begin
    U01: TIMER generic map( CYCLE=>6750000 ) -- 250 ms
           port map( CLK=>CLK, PULSE=>TOUT );

    U02: BIN4 port map( SW0=>TOUT, LED0=>INDAT(0),
    LED1=>INDAT(1), LED2=>INDAT(2), LED3=>INDAT(3) ); 

    LEDS <= LEDPAT( conv_integer(INDAT)); -- read ROM data
        LED0 <= LEDS(0); LED1 <= LEDS(1);
        LED2 <= LEDS(2); LED3 <= LEDS(3);   
        LED4 <= LEDS(4); LED5 <= LEDS(5);
        LED6 <= LEDS(6); LED7 <= LEDS(7); 
end RTL;
