library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;

entity MUSICBOX is
port( CLK:   in   std_logic;
      RESET: in std_logic;
      SP:     out  std_logic );
end MUSICBOX;

architecture RTL of MUSICBOX is

component SOUND
port( CLK:      in  std_logic;
      INTERVAL: in  integer range 0 to 25813; -- ontei
      LD:       in  std_logic;
      RESET:    in  std_logic;
      SP:       out std_logic );
end component;

component LENGTH
port( CLK:   in  std_logic;
      LD:    in  std_logic;
      DATA:  in  std_logic_vector(3 downto 0);
      RESET: in  std_logic;
      TEMPO: out std_logic );
end component;

component SCORE_ROM
   port( ADDR: in  std_logic_vector( 7 downto 0 );
         DATA: out std_logic_vector( 15 downto 0 ) ); 
end component;

type    MY_STATE is   ( FETCH, DECODE, EXECUTE, WAITS );
signal  MODE:	MY_STATE;

type VALARRAY is array( 0 to 47 ) of integer range 0 to 25831;
constant INTVAL: VALARRAY := (
--  Do     --     Re     --     Mi     Fa     --     So     --     Ra     --     Si     Do
  25813, 24369, 22998, 21705, 20485, 19341, 18243, 17219, 16245, 15341, 14486, 13664, 12907, 0, 0, 0,
  12907, 12185, 11499, 10853, 10243,  9671,  9122,  8610,  8123,  7670,  7243,  6832,  6453, 0, 0, 0,
   6453,  6092,  5750,  5426,  5121,  4835,  4561,  4305,  4061,  3835,  3621,  3417,  3417, 0, 0, 0 );

signal PC:    std_logic_vector(  7 downto 0 ); -- Program Counter
signal IR:    std_logic_vector( 15 downto 0 ); -- Instruction Register
signal ROMOUT: std_logic_vector( 15 downto 0 );
signal INST:  std_logic_vector(  3 downto 0 ); -- Instruction
signal OP1:   std_logic_vector(  3 downto 0 ); -- Operend 1
signal OP2:   std_logic_vector(  7 downto 0 ); -- Operand 2
signal VAL:   integer range 0 to 25813;        -- ontei
signal LEN:   std_logic_vector(  3 downto 0 ); -- nagasa
signal WFLAG: std_logic;                       -- nagasa seigyo
signal LD:    std_logic;                       -- OP latch timing

begin
U01: SOUND     port map(CLK=>CLK, INTERVAL=>VAL, LD=>LD, RESET=>RESET, SP=>SP);
U02: LENGTH    port map(CLK=>CLK, LD=>LD, DATA=>LEN, RESET=>LD, TEMPO=>WFLAG);
U03: SCORE_ROM port map(ADDR=>PC, DATA=>ROMOUT);

STATE_MACHINE: process( RESET, CLK ) is
begin
    if( RESET = '0' ) then MODE <= FETCH; PC <= (others=>'0');
    elsif( CLK'event and CLK='0' ) then
       case MODE is
           when FETCH   => MODE <= DECODE;
           when DECODE  => MODE <= EXECUTE;
           when EXECUTE => MODE <= WAITS;
           when WAITS  => 
               if ( INST = "1000" ) then PC <= (others=>'0'); MODE <= FETCH; --Loop
               elsif( WFLAG = '0' ) then PC <= PC + 1;        MODE <= FETCH;
               end if;
       end case;
    end if;
end process;

FETCH_state: process( RESET, CLK ) is
begin
    if( RESET = '0' ) then IR <=(others=>'0');
    elsif( CLK'event and CLK = '0' ) then
        if( MODE = FETCH )then IR <= ROMOUT;
        end if;
    end if;
end process;

DECODE_state: process( RESET, CLK ) is
begin
    if( RESET = '0' ) then INST <= "0000";
    elsif( CLK'event and CLK = '0' ) then
        if( MODE = DECODE )then
            INST <= IR( 15 downto 12 );
            OP1  <= IR( 11 downto  8 );
            OP2  <= IR(  7 downto  0 );
        end if;
    end if;
end process;

EXECUTE_state: process( RESET, CLK ) is
begin
    if( RESET = '0' ) then LD <= '1';
    elsif( CLK'event and CLK = '0') then LD <= '1';
        if( MODE = EXECUTE )then
            case INST is
                WHEN "0000" => null;                  --NOP  
                WHEN "0001" =>                        --NOTE
                    LEN <= OP1; VAL <= INTVAL( conv_integer( OP2 )); LD <= '0';
                WHEN "0010" =>                        --REST
                    LEN <= OP1; VAL <= 0; LD <= '0';
                WHEN "1000" => null;                  --LOOP
                WHEN "1111" => null;                  --HALT
                WHEN others => null;
            end case;
        end if;
    end if;
end process;
end RTL;
